from os import path
import numpy as np
import soundfile as sf
from scipy import signal as scipy_signal


def read_signal(audio_file):
    signal, samplerate = sf.read(audio_file)
    return signal, samplerate


def write_signal(signal, audio_file_with_extension, samplerate, overwrite=False):
    if path.exists(audio_file_with_extension) and not overwrite:
        return False
    sf.write(audio_file_with_extension, signal, samplerate)
    return True


def normalise(signal):
    return signal / np.abs(np.max(signal))


def stereo_to_mono(signal):
    assert signal.shape[1] == 2
    return normalise(signal[:, 0] + signal[:, 1])


def convolve_ir_to_signal(signal, signal_samplerate, ir, ir_samplerate, out_samplerate):
    if not any([isinstance(signal, np.array), isinstance(signal, str)]):
        raise TypeError('Signal must be either an np.array or a path to an \
                        audio file')
    if not any([isinstance(ir, np.array), isinstance(ir, str)]):
        raise TypeError('Ir must be either an np.array or a path to an \
                        audio file')
    f = signal.copy()
    g = ir.copy()
    if f.shape[1] > 1:
        f = stereo_to_mono(f)
    if g.shape[1] > 1:
        g = stereo_to_mono(g)

    t = np.max(f.shape) / ir_samplerate
    f = scipy_signal.resample(f, int(np.floor(t * out_samplerate)))
    f = normalise(f)
    t = np.max(g.shape) / signal_samplerate
    g = scipy_signal.resample(g, int(np.floor(t * out_samplerate)))
    g = normalise(g)
    y = np.convolve(g, f, 'full')
    y = normalise(y)
    return y




import tensorflow as tf

class PatchClassifier(tf.keras.Model):
    def __init__(self, n_classes, batch_size, input_shape, max_pool=False):
        super().__init__()
        input_tensor = tf.keras.layers.Input(shape=input_shape, batch_size=batch_size)
        self.backbone = tf.keras.applications.ResNet50(include_top=False,
                                                       weights='imagenet',
                                                       input_tensor=input_tensor,
                                                       pooling=None)
        if max_pool:
            self.max_pool = tf.keras.layers.MaxPooling2D(pool_size=(1, 1), strides=None)
        else:
            max_pool = None
        self.flatten = tf.keras.layers.Flatten()
        self.dropout = tf.keras.layers.Dropout(0.5)
        self.dense = tf.keras.layers.Dense(units=n_classes, activation='softmax')

    def call(self, inputs, training=False):
        x = self.backbone(inputs)
        if self.max_pool:
            x = self.max_pool(x)
        x = self.flatten(x)
        if training:
            x = self.dropout(x)
        return self.dense(x)




import cv2


def read_image(image_path, resize=None, normalise=False):
    image = cv2.imread(image_path, cv2.IMREAD_UNCHANGED)

    if image is None:
        raise TypeError
    if image.size == 0:
        raise TypeError

    if image.shape[-1] == 4:
        image = cv2.cvtColor(image, cv2.COLOR_BGRA2RGBA)
    else:
        image = cv2.cvtColor(image, cv2.COLOR_BGR2RGBA)

    if resize:
        image = cv2.resize(image, resize)
    if normalise and image[:, :, 0].mean() > 1:
        image /= 255
    return image


def show_image(image):
    image = cv2.cvtColor(image, cv2.COLOR_RGB2BGR)
    cv2.imshow('image', image)
    while cv2.waitKey(10) != ord('q'):
        pass


def show_images(images):
    for i, image in enumerate(images):
        cv2.imshow('image{}'.format(i), image)
    while cv2.waitKey(10) != ord('q'):
        pass



import requests
from os import path, makedirs, unlink


def download_from_google_drive(gdrive_file_id, dest_path, overwrite=False) -> bool:
    URL = "https://drive.google.com/uc?export=download"
    session = requests.Session()
    response = session.get(URL, params={'id':gdrive_file_id}, stream=True)
    token = get_confirm_token(response)
    if token:
        params = {'id':gdrive_file_id, 'confirm':token}
        response = session.get(URL, params=params, stream=True)
    else:
        return False
    return save_response_content(response, dest_path, overwrite)    


def get_confirm_token(response):
    for key, value in response.cookies.items():
        if key.startswith('download_warning'):
            return value
    return None


def save_response_content(response, dest_path, overwrite=False):
    if path.isdir(dest_path):
        return False
    if path.exists(dest_path) and not overwrite:
        return False

    if path.exists(dest_path):
        unlink(dest_path)
    else:
        if not path.exists(path.dirname(dest_path)):
            makedirs(path.dirname(dest_path))

    chunks = 32768
    with open(dest_path, "wb") as f:
        for chunk in response.iter_content(chunks):
            if chunk:
                f.write(chunk)
    return True

